#include "mouse-panel.h"

struct _MousePanel
{
    Panel parent_instance;

    GtkImage* mouse_image;
    GtkLabel* mouse_name;
    AdwPreferencesGroup* dpi_preference_group;
};

G_DEFINE_TYPE (MousePanel, mouse_panel, panel_get_type())

static void
mouse_panel_class_init(MousePanelClass* klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS(klass);
	GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(klass);
	PanelClass* panel_class = GINPUT_PANEL_CLASS(klass);

	gtk_widget_class_set_template_from_resource(widget_class, "/v/ginput/panel-mouse.ui");

    gtk_widget_class_bind_template_child(widget_class, MousePanel, mouse_image);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, mouse_name);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, dpi_preference_group);
}

static void
mouse_panel_init(MousePanel* self)
{
	gtk_widget_init_template(GTK_WIDGET(self));
}

Panel*
mouse_panel_new(void)
{
    return g_object_new(mouse_panel_get_type(), NULL);
}

void mouse_panel_set_device(MousePanel* self, device_t* device)
{
    // Set device image
    char* image = device_get_image(device);
    gtk_image_set_from_file(self->mouse_image, image);
    free(image);

    // Set device name
    gtk_label_set_label(self->mouse_name, device_get_name(device));
    
    // Set mouse dpi
    struct MOUSE_DPI_LEVELS dpi;
    int (*driver_mouse_dpi_get)(void*, void*) = dlsym(device_driver(device), "driver_mouse_dpi_get");
    int dpi_res = driver_mouse_dpi_get(device_handle(device), &dpi);
    if(!dpi_res)
    {
        // Set dpi list
        for(size_t i = 0; i < dpi.level_count; i++)
        {
            AdwPreferencesRow* row = ADW_PREFERENCES_ROW(adw_preferences_row_new());
            
            // Convert dpi int to string
            char dpi_str[10];
            sprintf(dpi_str, "%u", dpi.level[i].dpi_x);

            GtkLabel* label = GTK_LABEL(gtk_label_new(dpi_str));
            gtk_widget_add_css_class(GTK_WIDGET(label), "heading");
            gtk_list_box_row_set_child(GTK_LIST_BOX_ROW(row), GTK_WIDGET(label));

            adw_preferences_group_add(self->dpi_preference_group, GTK_WIDGET(row));
        }
    }
}
